clear
clear global
tic
%MANUAL:
%All data and subject specific information is dealt with in the three files: 1) loaddata (loading of constants);
%2)subjectdata(loading of subject specific data 3)subjectinfo (additional information to be read from data)


%CONSTANT: NEED TO BE SPECIFIED BY USER ONCE; AFTER THAT THEY HAVE TO REMAIN CONSTANT
global model np sr                            %Model specific: Model specifier, number of parameters
global lba uba lbb1 ubb1 lbc ubc 	%Bounds for rescaling of raster=Bounds for parameters of the model
global data nt ns                           %Data specific: data, columns, number of trials, number of subjects

%VARIABLE DATA:
global rp                                   %current point in raster from which the search starts
global rpc1 rpc2 rpc4                  %raster point coordinates; just used for passing on the 
global countdecks

% global deck

%Initialization of some variables needed for recursions; storage for results obtained by loope1
parmvc=[];				    %initialize collection of parameter matrices from all raster points
chivc=[];				    %initialize collection of chi vectors from all raster points
sm=[];                      %initialize collection of subject matrices from all raster points
maxchivc=[];				%initialize
maxparmvc=[];				%initialize


%-----------------------DEFINE CONSTANT GLOBAL VARIABLE---------------------
%Model specific constants
model = 7;				%model specification
np = 3;					%number of parameters
%Data specific: loaddata file NEEDS CHANGE IF DATA CHANGES
loaddata
%Bounds for Rescaling: CONSTANT
%Lower and upper bounds of parameters a, b1, b2, and c for the rescaling process
lba=0;	uba=1;% learning rate or recency 
lbb1=0; ubb1=1;% relative attention wgt to loss 
lbc=-5; ubc=5; % sensitivity 

%----------------------------------------RASTER---------------------------------
%Creation of raster
p1a = [0.15 0.5 0.85];  % learning rate and recency
p1b1 = [0.15 0.5 0.85]; % relative attention wgt to losses
p1c = [-1 0 1];  % sensitivity
sr=size(p1a,2); 	%size of raster; raster has sr^np points


%Rescale raster to be used by minsearch
for pi = 1:sr   
	p0a(pi)  = scale(uba,lba,p1a(pi));    
  	p0b1(pi) = scale(ubb1,lbb1,p1b1(pi));  
    p0c(pi)  = scale(ubc,lbc,p1c(pi));    
end

%---------------------------------------Model----------------------------------
'Begin model with np parameters'
model
np

%For each point in the raster the optimization process is done here 
if np==3 %error check
    for rpc1 = 1:sr
        for rpc2 = 1:sr
            for rpc4 = 1:sr
                    rp=[p0a(rpc1) p0b1(rpc2) p0c(rpc4)]' %raster point in rescaled coordinates
                    
                    %MAIN PART
                    [parmv1, chiv1]=loope1; %Result of the optimization process starting at this raster point
                    %chiv1 is a column vector with ns entries
                    %parmv1 is a ns x np matrix; one row per particant; one column per parameter
                    
                    parmvc=[parmvc parmv1];	%Collects all parameter-matrices via horizontal concatination from each of the raster points
                    
                    chivc=[chivc chiv1];	%Collects all chi-column-vectors via horizontal concatination from each of the raster points                    
                    save gbt1mod7p4cnp chivc parmvc;   %Storing; in case the program crashes
            end
        end
    end
else
    'Error: number of parameters does not fit model'
end
  
%------------------------------------Loading of best parameters--------------------------
mm=size(chivc,1); %should be equal to the number of subjects 
   for m = 1:mm
       
       %Find best chi for subject m
       [c,i] = max (chivc(m,:));		%look for the best chi for subject m in the chi vectors for all raster points
       maxchivc = [maxchivc; c];        %add the best chi to the maxchi column vector (one entry per subject)                       
       
       %Find the parameters that produced the best chi
       j = ((i-1)*np+1):((i-1)*np+np);  %vector containing the indices where to find the paramters for the best chi
       maxparmvc=[maxparmvc; parmvc(m,j)];	%Collects parameters with best chi
       %maxparmvc will have one row per subject and one column per parameter
   end 
   
   %Rescaling the returned parameters of minsearch to the model parameters; same as in fitr1
   for m = 1:mm
       tmaxparmvc(m,1) = scalei(uba,lba,maxparmvc(m,1));
       tmaxparmvc(m,2) = scalei(ubb1,lbb1,maxparmvc(m,2));
       tmaxparmvc(m,3) = scalei(ubc,lbc,maxparmvc(m,3));           
   end
   
   %Loads additionals information about the subjects
   for n=1:ns
       sv=subjectinfo(n);   %returns subjectinfo vector (row)
       sm=[sm; sv];         %collects all column subject vectors into a matrix with one column per subject
       %sm will have one row per subject and one column per additional information about the subject    
   end
   
res = [sm maxchivc tmaxparmvc]
%res will have one row per participant
                                  % for unconstrain   
save results1  sm maxchivc tmaxparmvc;
save results2.txt  res -ascii

'End model with np parameters'
model
np
toc

